@@ -1,56 +1,25 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the Qt Virtual Keyboard module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 or (at your option) any later version
-** approved by the KDE Free Qt Foundation. The licenses are as published by
-** the Free Software Foundation and appearing in the file LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QtVirtualKeyboard/qvirtualkeyboardinputcontext.h>
 #include <QtVirtualKeyboard/private/qvirtualkeyboardinputcontext_p.h>
 #include <QtVirtualKeyboard/private/shifthandler_p.h>
 #include <QtVirtualKeyboard/private/platforminputcontext_p.h>
 #include <QtVirtualKeyboard/private/virtualkeyboarddebug_p.h>
+#include <QtVirtualKeyboard/qvirtualkeyboardobserver.h>
 
 #include <QTextFormat>
 #include <QGuiApplication>
-
-/*!
-    \namespace QtVirtualKeyboard
-    \inmodule QtVirtualKeyboard
-
-    \brief Namespace for the Qt Virtual Keyboard C++ API.
-*/
+#include <QtGui/private/qhighdpiscaling_p.h>
 
 QT_BEGIN_NAMESPACE
 using namespace QtVirtualKeyboard;
 
 /*!
     \qmltype InputContext
-    \instantiates QVirtualKeyboardInputContext
+    \nativetype QVirtualKeyboardInputContext
     \inqmlmodule QtQuick.VirtualKeyboard
-    \ingroup qtvirtualkeyboard-qml
+    \ingroup qtvirtualkeyboard-internal-qml
     \brief Provides access to an input context.
 
     The InputContext can be accessed as singleton instance.
@@ -59,6 +28,7 @@ using namespace QtVirtualKeyboard;
 /*!
     \class QVirtualKeyboardInputContext
     \inmodule QtVirtualKeyboard
+    \ingroup qtvirtualkeyboard-cpp-for-devs
     \brief Provides access to an input context.
 */
 
@@ -77,6 +47,7 @@ QVirtualKeyboardInputContext::QVirtualKeyboardInputContext(QObject *parent) :
     QObject::connect(d->_shiftHandler, &ShiftHandler::capsLockActiveChanged, this, &QVirtualKeyboardInputContext::capsLockActiveChanged);
     QObject::connect(d->_shiftHandler, &ShiftHandler::uppercaseChanged, this, &QVirtualKeyboardInputContext::uppercaseChanged);
     QObject::connect(d, &QVirtualKeyboardInputContextPrivate::localeChanged, this, &QVirtualKeyboardInputContext::localeChanged);
+    QObject::connect(d, &QVirtualKeyboardInputContextPrivate::inputItemChanged, this, &QVirtualKeyboardInputContext::inputItemChanged);
 }
 
 /*!
@@ -137,7 +108,7 @@ void QVirtualKeyboardInputContext::setPreeditText(const QString &text, QList<QIn
         if (!d->testAttribute(attributes, QInputMethodEvent::TextFormat)) {
             QTextCharFormat textFormat;
             textFormat.setUnderlineStyle(QTextCharFormat::SingleUnderline);
-            attributes.append(QInputMethodEvent::Attribute(QInputMethodEvent::TextFormat, 0, text.length(), textFormat));
+            attributes.append(QInputMethodEvent::Attribute(QInputMethodEvent::TextFormat, 0, text.size(), textFormat));
         }
     } else if (d->_forceCursorPosition != -1) {
         d->addSelectionAttribute(attributes);
@@ -199,6 +170,12 @@ QString QVirtualKeyboardInputContext::locale() const
     return d->locale();
 }
 
+QObject *QVirtualKeyboardInputContext::inputItem() const
+{
+    Q_D(const QVirtualKeyboardInputContext);
+    return d->inputItem();
+}
+
 QVirtualKeyboardInputEngine *QVirtualKeyboardInputContext::inputEngine() const
 {
     Q_D(const QVirtualKeyboardInputContext);
@@ -218,7 +195,7 @@ QVirtualKeyboardInputEngine *QVirtualKeyboardInputContext::inputEngine() const
 void QVirtualKeyboardInputContext::sendKeyClick(int key, const QString &text, int modifiers)
 {
     Q_D(QVirtualKeyboardInputContext);
-    if (d->_focus && d->platformInputContext) {
+    if ((d->_focus && d->platformInputContext) || QtVirtualKeyboard::forceEventsWithoutFocus()) {
         QKeyEvent pressEvent(QEvent::KeyPress, key, Qt::KeyboardModifiers(modifiers), text);
         QKeyEvent releaseEvent(QEvent::KeyRelease, key, Qt::KeyboardModifiers(modifiers), text);
         VIRTUALKEYBOARD_DEBUG().nospace() << "InputContext::sendKeyClick()"
@@ -231,6 +208,10 @@ void QVirtualKeyboardInputContext::sendKeyClick(int key, const QString &text, in
         d->setState(QVirtualKeyboardInputContextPrivate::State::KeyEvent);
         d->platformInputContext->sendKeyEvent(&pressEvent);
         d->platformInputContext->sendKeyEvent(&releaseEvent);
+
+        if (key == Qt::Key_Return || key == Qt::Key_Enter)
+            d->maybeCloseOnReturn();
+
         if (d->activeKeys.isEmpty())
             d->clearState(QVirtualKeyboardInputContextPrivate::State::KeyEvent);
     } else {
@@ -264,9 +245,9 @@ void QVirtualKeyboardInputContext::commit()
 
     Commits the final \a text to the input item and optionally
     modifies the text relative to the start of the pre-edit text.
-    If \a replaceFrom is non-zero, the \a text replaces the
-    contents relative to \a replaceFrom with a length of
-    \a replaceLength.
+    If \e replaceFrom is non-zero, the \a text replaces the
+    contents relative to \e replaceFrom with a length of
+    \e replaceLength.
 */
 /*!
     Commits the final \a text to the input item and optionally
@@ -336,36 +317,103 @@ void QVirtualKeyboardInputContext::clear()
 */
 void QVirtualKeyboardInputContext::setSelectionOnFocusObject(const QPointF &anchorPos, const QPointF &cursorPos)
 {
-    QPlatformInputContext::setSelectionOnFocusObject(anchorPos, cursorPos);
+    QWindow *window = qApp->focusWindow();
+    const QPointF &nativeAnchorPos = QHighDpi::toNativePixels(anchorPos, window);
+    const QPointF &nativeCursorPos = QHighDpi::toNativePixels(cursorPos, window);
+
+    QPlatformInputContext::setSelectionOnFocusObject(nativeAnchorPos, nativeCursorPos);
 }
 
+/*!
+    \property QVirtualKeyboardInputContext::anchorRectIntersectsClipRect
+    \brief Holds \c true if the bounding rectangle of the selection anchor
+    intersects the exposed input item rectangle.
+
+    \sa Qt::ImAnchorRectangle, Qt::ImInputItemClipRectangle
+*/
+/*!
+    \qmlproperty bool InputContext::anchorRectIntersectsClipRect
+    \readonly
+    \brief Holds \c true if the bounding rectangle of the selection anchor
+    intersects the exposed input item rectangle.
+
+    \sa Qt::ImAnchorRectangle, Qt::ImInputItemClipRectangle
+*/
 bool QVirtualKeyboardInputContext::anchorRectIntersectsClipRect() const
 {
     Q_D(const QVirtualKeyboardInputContext);
     return d->anchorRectIntersectsClipRect;
 }
 
+/*!
+    \property QVirtualKeyboardInputContext::cursorRectIntersectsClipRect
+    \brief Holds \c true if the bounding rectangle of the input cursor
+    intersects the exposed input item rectangle.
+
+    \sa Qt::ImCursorRectangle, Qt::ImInputItemClipRectangle
+*/
+/*!
+    \qmlproperty bool InputContext::cursorRectIntersectsClipRect
+    \readonly
+    \brief Holds \c true if the bounding rectangle of the input cursor
+    intersects the exposed input item rectangle.
+
+    \sa Qt::ImCursorRectangle, Qt::ImInputItemClipRectangle
+*/
 bool QVirtualKeyboardInputContext::cursorRectIntersectsClipRect() const
 {
     Q_D(const QVirtualKeyboardInputContext);
     return d->cursorRectIntersectsClipRect;
 }
 
+/*!
+    \property QVirtualKeyboardInputContext::selectionControlVisible
+    \brief Holds \c true if the selection control is currently visible.
+*/
+/*!
+    \qmlproperty bool InputContext::selectionControlVisible
+    \readonly
+    \brief Holds \c true if the selection control is currently visible.
+*/
 bool QVirtualKeyboardInputContext::isSelectionControlVisible() const
 {
     Q_D(const QVirtualKeyboardInputContext);
     return d->selectionControlVisible;
 }
 
+/*!
+    \internal
+*/
 QVirtualKeyboardInputContextPrivate *QVirtualKeyboardInputContext::priv() const
 {
     Q_D(const QVirtualKeyboardInputContext);
     return const_cast<QVirtualKeyboardInputContextPrivate *>(d);
 }
 
+/*!
+    \property QVirtualKeyboardInputContext::keyboardObserver
+    \since QtQuick.VirtualKeyboard 6.1
+    \brief Holds the keyboard observer object, which can be used to receive
+    notifications of keyboard change events.
+*/
+/*!
+    \qmlproperty KeyboardObserver InputContext::keyboardObserver
+    \readonly
+    \since QtQuick.VirtualKeyboard 6.1
+    \brief Holds the keyboard observer object, which can be used to receive
+    notifications of keyboard change events.
+*/
+QVirtualKeyboardObserver *QVirtualKeyboardInputContext::keyboardObserver() const
+{
+    Q_D(const QVirtualKeyboardInputContext);
+    return d->keyboardObserver;
+}
+
 /*!
     \qmlproperty bool InputContext::shift
-    \deprecated Use \l shiftActive instead.
+    \deprecated
+
+    Use \l shiftActive instead.
 
     This property is changed when the shift status changes.
 */
@@ -373,7 +421,9 @@ QVirtualKeyboardInputContextPrivate *QVirtualKeyboardInputContext::priv() const
 /*!
     \property QVirtualKeyboardInputContext::shift
     \brief the shift status.
-    \deprecated Use \l shiftActive instead.
+    \deprecated
+
+    Use \l shiftActive instead.
 
     This property is changed when the shift status changes.
 */
@@ -394,7 +444,9 @@ QVirtualKeyboardInputContextPrivate *QVirtualKeyboardInputContext::priv() const
 
 /*!
     \qmlproperty bool InputContext::capsLock
-    \deprecated Use \l capsLockActive instead.
+    \deprecated
+
+    Use \l capsLockActive instead.
 
     This property is changed when the caps lock status changes.
 */
@@ -402,7 +454,9 @@ QVirtualKeyboardInputContextPrivate *QVirtualKeyboardInputContext::priv() const
 /*!
     \property QVirtualKeyboardInputContext::capsLock
     \brief the caps lock status.
-    \deprecated Use \l capsLockActive instead.
+    \deprecated
+
+    Use \l capsLockActive instead.
 
     This property is changed when the caps lock status changes.
 */
@@ -569,6 +623,21 @@ QVirtualKeyboardInputContextPrivate *QVirtualKeyboardInputContext::priv() const
     This property is changed when the input locale changes.
 */
 
+/*!
+    \qmlproperty QtObject InputContext::inputItem
+    \deprecated
+
+    This property is changed when the focused input item changes.
+*/
+
+/*!
+    \property QVirtualKeyboardInputContext::inputItem
+    \brief the focused input item.
+    \deprecated
+
+    This property is changed when the focused input item changes.
+*/
+
 /*!
     \qmlproperty InputEngine InputContext::inputEngine
 
@@ -582,4 +651,8 @@ QVirtualKeyboardInputContextPrivate *QVirtualKeyboardInputContext::priv() const
     This property stores the input engine.
 */
 
+/*!
+    \property QVirtualKeyboardInputContext::priv
+    \internal
+*/
 QT_END_NAMESPACE
